' ---------------------------------------------------------------
' Copyright 2006, SAS Institute Inc.
' ---------------------------------------------------------------
Imports SAS.Shared.AddIns
Imports System.Text
Imports System.Xml
Imports System.IO

Namespace SASVB.Compare
    Public Class Compare
        Implements SAS.Shared.AddIns.ISASTaskAddIn, SAS.Shared.AddIns.ISASTaskDescription, SAS.Shared.AddIns.ISASTask

        Private _consumer As ISASTaskConsumer

#Region "Task state and properties"

        Private _referenceVar As String = ""
        Private _withVar As String = ""
        Private _briefSummary As Boolean = False
        Private _allStats As Boolean = False

        Public Property ReferenceVar() As String
            Set(ByVal Value As String)
                _referenceVar = Value
            End Set
            Get
                Return _referenceVar
            End Get
        End Property

        Public Property WithVar() As String
            Set(ByVal Value As String)
                _withVar = Value
            End Set
            Get
                Return _withVar
            End Get
        End Property

        Public Property BriefSummary() As Boolean
            Set(ByVal Value As Boolean)
                _briefSummary = Value
            End Set
            Get
                Return _briefSummary
            End Get
        End Property

        Public Property AllStats() As Boolean
            Set(ByVal Value As Boolean)
                _allStats = Value
            End Set
            Get
                Return _allStats
            End Get
        End Property

#End Region

#Region "ISASTaskAddin"

        Public ReadOnly Property AddInDescription() As String Implements SAS.Shared.AddIns.ISASTaskAddIn.AddInDescription
            Get
                AddInDescription = "Compare two columns in a single data set."
            End Get
        End Property

        Public ReadOnly Property AddInName() As String Implements SAS.Shared.AddIns.ISASTaskAddIn.AddInName
            Get
                AddInName = "Compare Columns"
            End Get
        End Property

        Public Function Connect(ByVal inConsumer As SAS.Shared.AddIns.ISASTaskConsumer) As Boolean Implements SAS.Shared.AddIns.ISASTaskAddIn.Connect
            ' I need to call back into the consumer (host application), so I need to
            ' save the Consumer passed in for future use.
            Me._consumer = inConsumer
            Connect = True
        End Function

        Public Sub Disconnect() Implements SAS.Shared.AddIns.ISASTaskAddIn.Disconnect
            ' nothing special needed for this task
        End Sub

        Public WriteOnly Property Language() As String Implements SAS.Shared.AddIns.ISASTaskAddIn.Language
            Set(ByVal Value As String)
                ' nothing special needed for this task
            End Set
        End Property

        Public Function Languages(ByRef Items() As String) As Integer Implements SAS.Shared.AddIns.ISASTaskAddIn.Languages
            ' supporting only english in this task
            Items = New String() {"en-US"}
            Languages = 1
        End Function

        Public ReadOnly Property VisibleInManager() As Boolean Implements SAS.Shared.AddIns.ISASTaskAddIn.VisibleInManager
            Get
                VisibleInManager = True
            End Get
        End Property

#End Region

#Region "ISASTaskDescription"

        Public ReadOnly Property Clsid() As String Implements SAS.Shared.AddIns.ISASTaskDescription.Clsid
            Get
                Clsid = "4e9d4e6c-308b-43e8-af65-1cd7a14198c6"
            End Get
        End Property

        Public ReadOnly Property FriendlyName() As String Implements SAS.Shared.AddIns.ISASTaskDescription.FriendlyName
            Get
                FriendlyName = "Compare Columns"
            End Get
        End Property

        Public ReadOnly Property GeneratesListOutput() As Boolean Implements SAS.Shared.AddIns.ISASTaskDescription.GeneratesListOutput
            Get
                GeneratesListOutput = True
            End Get
        End Property

        Public ReadOnly Property GeneratesSasCode() As Boolean Implements SAS.Shared.AddIns.ISASTaskDescription.GeneratesSasCode
            Get
                GeneratesSasCode = True
            End Get
        End Property

        Public ReadOnly Property IconAssembly() As String Implements SAS.Shared.AddIns.ISASTaskDescription.IconAssembly
            Get
                ' return this assembly as the icon assembly
                IconAssembly = System.Reflection.Assembly.GetExecutingAssembly.Location
            End Get
        End Property

        Public ReadOnly Property IconName() As String Implements SAS.Shared.AddIns.ISASTaskDescription.IconName
            Get
                IconName = "SASVB.CustomTasks.compare.ico"
            End Get
        End Property

        Public ReadOnly Property MajorVersion() As Integer Implements SAS.Shared.AddIns.ISASTaskDescription.MajorVersion
            Get
                MajorVersion = 1
            End Get
        End Property

        Public ReadOnly Property MinorVersion() As Integer Implements SAS.Shared.AddIns.ISASTaskDescription.MinorVersion
            Get
                MinorVersion = 0
            End Get
        End Property

        Public ReadOnly Property NumericColsRequired() As Integer Implements SAS.Shared.AddIns.ISASTaskDescription.NumericColsRequired
            Get
                NumericColsRequired = 0
            End Get
        End Property

        Public ReadOnly Property ProcsUsed() As String Implements SAS.Shared.AddIns.ISASTaskDescription.ProcsUsed
            Get
                ProcsUsed = "COMPARE"
            End Get
        End Property

        Public ReadOnly Property ProductsOptional() As String Implements SAS.Shared.AddIns.ISASTaskDescription.ProductsOptional
            Get
                ProductsOptional = ""
            End Get
        End Property

        Public ReadOnly Property ProductsRequired() As String Implements SAS.Shared.AddIns.ISASTaskDescription.ProductsRequired
            Get
                ProductsRequired = "BASE"
            End Get
        End Property

        Public ReadOnly Property RequiresData() As Boolean Implements SAS.Shared.AddIns.ISASTaskDescription.RequiresData
            Get
                RequiresData = True
            End Get
        End Property

        Public ReadOnly Property StandardCategory() As Boolean Implements SAS.Shared.AddIns.ISASTaskDescription.StandardCategory
            Get
                StandardCategory = False
            End Get
        End Property

        Public ReadOnly Property TaskCategory() As String Implements SAS.Shared.AddIns.ISASTaskDescription.TaskCategory
            Get
                TaskCategory = Global.Category
            End Get
        End Property

        Public ReadOnly Property TaskDescription() As String Implements SAS.Shared.AddIns.ISASTaskDescription.TaskDescription
            Get
                TaskDescription = "Compares two columns in a single data set"
            End Get
        End Property

        Public ReadOnly Property TaskName() As String Implements SAS.Shared.AddIns.ISASTaskDescription.TaskName
            Get
                TaskName = "Compare Columns"
            End Get
        End Property

        Public ReadOnly Property TaskType() As SAS.Shared.AddIns.ShowType Implements SAS.Shared.AddIns.ISASTaskDescription.TaskType
            Get
                TaskType = ShowType.Wizard
            End Get
        End Property

        Public ReadOnly Property Validation() As String Implements SAS.Shared.AddIns.ISASTaskDescription.Validation
            Get
                Validation = "SAS-supplied example"
            End Get
        End Property

        Public ReadOnly Property WhatIsDescription() As String Implements SAS.Shared.AddIns.ISASTaskDescription.WhatIsDescription
            Get
                WhatIsDescription = "Compare two columns in a single data set"
            End Get
        End Property

#End Region

#Region "ISASTask"

        Public Function Initialize() As Boolean Implements SAS.Shared.AddIns.ISASTask.Initialize
            Initialize = True
        End Function

        Public Property Label() As String Implements SAS.Shared.AddIns.ISASTask.Label
            Get
                Label = "Compare Columns"
            End Get
            Set(ByVal Value As String)

            End Set
        End Property

        Public ReadOnly Property OutputDataCount() As Integer Implements SAS.Shared.AddIns.ISASTask.OutputDataCount
            Get
                OutputDataCount = 0
            End Get
        End Property

        Public Function OutputDataInfo(ByVal Index As Integer, ByRef Source As String, ByRef Label As String) As SAS.Shared.AddIns.OutputData Implements SAS.Shared.AddIns.ISASTask.OutputDataInfo
            Source = Nothing
            Label = Nothing
            OutputDataInfo = OutputData.Unknown
        End Function

        Public ReadOnly Property RunLog() As String Implements SAS.Shared.AddIns.ISASTask.RunLog
            Get
                RunLog = Nothing
            End Get
        End Property

        Public ReadOnly Property SasCode() As String Implements SAS.Shared.AddIns.ISASTask.SasCode
            Get
                Dim code As New System.Text.StringBuilder

                ' generate the PROC COMPARE code
                code.AppendFormat("PROC COMPARE DATA={0}.{1}{2}", _consumer.ActiveData.Library, _consumer.ActiveData.Member, Environment.NewLine)
                If _briefSummary = True Then code.Append(" BRIEFSUMMARY" + Environment.NewLine)
                If _allStats = True Then code.Append(" ALLSTATS" + Environment.NewLine)
                code.Append(" ;")
                code.AppendFormat("VAR {0};{1}", _referenceVar, Environment.NewLine)
                code.AppendFormat("WITH {0};{1}", _withVar, Environment.NewLine)
                code.Append("RUN;" + Environment.NewLine)

                Return code.ToString()
            End Get
        End Property

        Public Function Show(ByVal Owner As System.Windows.Forms.IWin32Window) As SAS.Shared.AddIns.ShowResult Implements SAS.Shared.AddIns.ISASTask.Show

            ' show the form
            Dim form1 As New CompareForm
            form1.Consumer = Me._consumer
            form1.TaskModel = Me
            If form1.ShowDialog(Owner) = Windows.Forms.DialogResult.OK Then
                Return ShowResult.RunNow
            Else : Return ShowResult.Canceled
            End If

        End Function

        Public Sub Terminate() Implements SAS.Shared.AddIns.ISASTask.Terminate
            ' nothing special needed for this task
        End Sub

        Public Property XmlState() As String Implements SAS.Shared.AddIns.ISASTask.XmlState
            Get
                Dim sw As StringWriter = New StringWriter
                Dim writer As XmlTextWriter = New XmlTextWriter(sw)
                writer.WriteStartElement("CompareColumns")
                writer.WriteElementString("ReferenceVariable", _referenceVar)
                writer.WriteElementString("WithVariable", _withVar)
                writer.WriteElementString("AllStats", XmlConvert.ToString(_allStats))
                writer.WriteElementString("BriefSummary", XmlConvert.ToString(_briefSummary))
                writer.WriteEndElement()
                writer.Close()

                XmlState = sw.ToString()

            End Get

            Set(ByVal Value As String)
                If Value <> Nothing And Value.Length > 0 Then
                    Try
                        Dim sr As StringReader = New StringReader(Value)
                        Dim reader As XmlTextReader = New XmlTextReader(sr)
                        reader.ReadStartElement("CompareColumns")
                        _referenceVar = reader.ReadElementString("ReferenceVariable")
                        _withVar = reader.ReadElementString("WithVariable")
                        _allStats = XmlConvert.ToBoolean(reader.ReadElementString("AllStats"))
                        _briefSummary = XmlConvert.ToBoolean(reader.ReadElementString("BriefSummary"))
                        reader.ReadEndElement()
                        reader.Close()
                    Catch
                    End Try

                End If

            End Set
        End Property

#End Region

    End Class

End Namespace