' --------------------------------------------------------------------------------
'  Copyright (c) 2006 by SAS Institute Inc, Cary NC 27513 USA 
'  
'  MergeDataTaskForm.vb
' 
'  Type:	VB.NET Source Code
'  Product: Custom Task for use in SAS Enterprise Guide or 
'           the SAS Add-In for Microsoft Office
' -------------------------------------------------------------------------------

#Region " Namespaces "
Imports System
Imports System.Xml          ' for XMLTextWriter and XMLTextReader
Imports System.IO           ' for StreamReader and StreamWriter
Imports SAS.Shared.AddIns   ' SAS AddIns namespace
Imports System.Text         ' for StringBuilder
Imports System.Collections  ' for ArrayList
#End Region

Namespace MergeData

    Public Class MergeDataTask
        Implements SAS.Shared.AddIns.ISASTask, SAS.Shared.AddIns.ISASTaskAddIn, SAS.Shared.AddIns.ISASTaskDescription


#Region " Private members for boilerplate values "

        Private sLabel As String = "Merge Data (VB.NET)"
        Private sAddInName As String = "Merge Data (VB.NET)"
        Private sAddInDescription As String = "Merge Data (VB.NET)"
        Private sProductsReq As String = "BASE"
        Private sProductsOpt As String = ""
        Private sTaskName = "Merge Data (VB.NET)"
        Private sCategory As String = "SAS Custom Task"
        Private sTaskDescription As String = "Merge Data (VB.NET)"
        Private sFriendlyName As String = "Merge Data (VB.NET)"
        Private sWhatIsDescription As String = "The Merge Data Custom Example Task uses the SAS DATA step to accomplish simple match merges.  This example is implemented with VB.NET."

        ' store reference to the application Consumer object
        Private consumer As ISASTaskConsumer = Nothing

#End Region


#Region " Constructor "

        ' keep a reference to the ISASTaskConsumer interface for the application
        Private _consumer As ISASTaskConsumer
        Private _label As String

        Public Sub New()

        End Sub

#End Region


#Region " ISASTask implementation "

        ' perform any initialization for the add-in, and return True
        Public Function Initialize() As Boolean Implements SAS.Shared.AddIns.ISASTask.Initialize
            Dim R As Random = New Random

            sOutputMemberName = "MERGEDATA" & R.Next(1000, 100000).ToString
            Return True
        End Function

        Public Property Label() As String Implements SAS.Shared.AddIns.ISASTask.Label
            Get
                Return sLabel
            End Get
            Set(ByVal Value As String)
                sLabel = Value
            End Set
        End Property

        ' if the task creates output data, indicate it here
        Public ReadOnly Property OutputDataCount() As Integer Implements SAS.Shared.AddIns.ISASTask.OutputDataCount
            Get
                ' Does this task create any output data sets?
                Return 1
            End Get
        End Property

        Public Function OutputDataInfo(ByVal Index As Integer, ByRef Source As String, ByRef Label As String) As SAS.Shared.AddIns.OutputData Implements SAS.Shared.AddIns.ISASTask.OutputDataInfo
            Source = _sOutputLibrary & "." & _sOutputMemberName
            Label = "Merged " & sOutputLibrary & "." & sOutputMemberName
            Return OutputData.Libref
        End Function

        Public ReadOnly Property RunLog() As String Implements SAS.Shared.AddIns.ISASTask.RunLog
            Get
                ' If your task does not generate SAS code, you can supply your own
                ' log text to record the work completed.
                Return ""
            End Get
        End Property

        Public ReadOnly Property SasCode() As String Implements SAS.Shared.AddIns.ISASTask.SasCode
            Get
                Dim code As StringBuilder

                Select Case MergeType
                    Case eMergeType.CommonVariable
                        code = GenerateCommonVariableMergeCode()
                    Case eMergeType.CommonVariableMergeOutputMatches
                        code = GenerateCommonVriableMergeOutputMatchesCode()
                    Case eMergeType.MasterAndTransaction
                        code = GenerateMasterAndTransactionCode()
                    Case Else
                        code = GenerateDatasetOrderCode()
                End Select

                Return code.ToString
            End Get
        End Property

        Public Function Show(ByVal Owner As System.Windows.Forms.IWin32Window) As SAS.Shared.AddIns.ShowResult Implements SAS.Shared.AddIns.ISASTask.Show
            ' show the form
            Dim mdf As New MergeDataTaskForm
            mdf.Text = sLabel
            mdf.Consumer = _consumer
            mdf.taskModel = Me

            If mdf.ShowDialog() = Windows.Forms.DialogResult.OK Then
                Return ShowResult.RunNow
            Else : Return ShowResult.Canceled
            End If
        End Function

        Public Sub Terminate() Implements SAS.Shared.AddIns.ISASTask.Terminate
            ' cleanup as needed
        End Sub

        ' Manage the state of the task when serializing to and from the project
        ' makes use of some helper methods: WriteXML and ReadXML
        Public Property XmlState() As String Implements SAS.Shared.AddIns.ISASTask.XmlState
            Get
                Return WriteXML()
            End Get
            Set(ByVal Value As String)
                ReadXML(Value)
            End Set
        End Property

#End Region


#Region " ISASTaskAddIn implementation "

        Public ReadOnly Property AddInDescription() As String Implements SAS.Shared.AddIns.ISASTaskAddIn.AddInDescription
            Get
                Return sAddInDescription
            End Get
        End Property

        Public ReadOnly Property AddInName() As String Implements SAS.Shared.AddIns.ISASTaskAddIn.AddInName
            Get
                Return sAddInName
            End Get
        End Property

        Public Function Connect(ByVal Consumer As SAS.Shared.AddIns.ISASTaskConsumer) As Boolean Implements SAS.Shared.AddIns.ISASTaskAddIn.Connect
            ' Save reference to the consumer application for use later
            Me._consumer = Consumer
            Connect = True
        End Function

        Public Sub Disconnect() Implements SAS.Shared.AddIns.ISASTaskAddIn.Disconnect
            ' Free our reference to the consumer application
            Me._consumer = Nothing
        End Sub

        Public WriteOnly Property Language() As String Implements SAS.Shared.AddIns.ISASTaskAddIn.Language
            Set(ByVal Value As String)
                ' handle different languages if necessary
            End Set
        End Property

        Public Function Languages(ByRef Items() As String) As Integer Implements SAS.Shared.AddIns.ISASTaskAddIn.Languages
            ' supporting only english in this task
            Items = New String() {"en-US"}
            Return 1
        End Function

        Public ReadOnly Property VisibleInManager() As Boolean Implements SAS.Shared.AddIns.ISASTaskAddIn.VisibleInManager
            Get
                Return True
            End Get
        End Property

#End Region


#Region " ISASTaskDescription implementation "

        Public ReadOnly Property Clsid() As String Implements SAS.Shared.AddIns.ISASTaskDescription.Clsid
            Get
                ' Return GUID value generated by template
                Return "8413E3D1-CE59-4574-A07A-A0DBC84FED55"
            End Get
        End Property

        Public ReadOnly Property FriendlyName() As String Implements SAS.Shared.AddIns.ISASTaskDescription.FriendlyName
            Get
                ' User-friendly name for the task
                Return sFriendlyName
            End Get
        End Property

        Public ReadOnly Property GeneratesListOutput() As Boolean Implements SAS.Shared.AddIns.ISASTaskDescription.GeneratesListOutput
            Get
                ' Does this task generate ODS-style output?
                Return True
            End Get
        End Property

        Public ReadOnly Property GeneratesSasCode() As Boolean Implements SAS.Shared.AddIns.ISASTaskDescription.GeneratesSasCode
            Get
                ' Return true if your task generates SAS program code
                Return True
            End Get
        End Property

        Public ReadOnly Property IconAssembly() As String Implements SAS.Shared.AddIns.ISASTaskDescription.IconAssembly
            Get
                ' return this assembly as the icon assembly
                Return System.Reflection.Assembly.GetExecutingAssembly().Location
            End Get
        End Property

        Public ReadOnly Property IconName() As String Implements SAS.Shared.AddIns.ISASTaskDescription.IconName
            Get
                ' Return the name of the icon within this assembly
                ' including namespace qualifiers
                Return "SASVB.CustomTasks.MergeDataTask.ico"
            End Get
        End Property

        Public ReadOnly Property MajorVersion() As Integer Implements SAS.Shared.AddIns.ISASTaskDescription.MajorVersion
            Get
                Return 1
            End Get
        End Property

        Public ReadOnly Property MinorVersion() As Integer Implements SAS.Shared.AddIns.ISASTaskDescription.MinorVersion
            Get
                Return 0
            End Get
        End Property

        Public ReadOnly Property NumericColsRequired() As Integer Implements SAS.Shared.AddIns.ISASTaskDescription.NumericColsRequired
            Get
                ' How many numeric variables are required in input data, if any?
                Return 0
            End Get
        End Property

        Public ReadOnly Property ProcsUsed() As String Implements SAS.Shared.AddIns.ISASTaskDescription.ProcsUsed
            Get
                ' What SAS procedures are used in this task?
                Return ""
            End Get
        End Property

        Public ReadOnly Property ProductsOptional() As String Implements SAS.Shared.AddIns.ISASTaskDescription.ProductsOptional
            Get
                ' What SAS products are optionally used by this task?
                Return sProductsOpt
            End Get
        End Property

        Public ReadOnly Property ProductsRequired() As String Implements SAS.Shared.AddIns.ISASTaskDescription.ProductsRequired
            Get
                ' What SAS products are required for this task to run?
                Return sProductsReq
            End Get
        End Property

        Public ReadOnly Property RequiresData() As Boolean Implements SAS.Shared.AddIns.ISASTaskDescription.RequiresData
            Get
                ' Does your task require input data from the application?
                Return False
            End Get
        End Property

        Public ReadOnly Property StandardCategory() As Boolean Implements SAS.Shared.AddIns.ISASTaskDescription.StandardCategory
            Get
                ' Used typically only by SAS-supplied tasks
                Return False
            End Get
        End Property

        Public ReadOnly Property TaskCategory() As String Implements SAS.Shared.AddIns.ISASTaskDescription.TaskCategory
            Get
                ' Task category
                Return sCategory
            End Get
        End Property

        Public ReadOnly Property TaskDescription() As String Implements SAS.Shared.AddIns.ISASTaskDescription.TaskDescription
            Get
                ' Description of your task
                Return sTaskDescription
            End Get
        End Property

        Public ReadOnly Property TaskName() As String Implements SAS.Shared.AddIns.ISASTaskDescription.TaskName
            Get
                ' The name of your task
                Return sTaskName
            End Get
        End Property

        Public ReadOnly Property TaskType() As SAS.Shared.AddIns.ShowType Implements SAS.Shared.AddIns.ISASTaskDescription.TaskType
            Get
                ' The type of your task (Code, Interactive, Wizard)
                Return SAS.Shared.AddIns.ShowType.Wizard
            End Get
        End Property

        Public ReadOnly Property Validation() As String Implements SAS.Shared.AddIns.ISASTaskDescription.Validation
            Get
                ' Add a validation string that makes sense for your organization
                ' For example, "dev", "test", or "prod"
                Return ""
            End Get
        End Property

        Public ReadOnly Property WhatIsDescription() As String Implements SAS.Shared.AddIns.ISASTaskDescription.WhatIsDescription
            Get
                ' Longer description for your task
                Return sWhatIsDescription
            End Get
        End Property

#End Region


#Region " Task state and Properties "
        Dim _sID As String
        Public Property sID() As String
            Get
                Return _sID
            End Get
            Set(ByVal Value As String)
                _sID = Value
            End Set
        End Property

        Dim _sOutputServer As String = "LOCAL"
        Public Property sOutputServer() As String
            Get
                Return _sOutputServer
            End Get
            Set(ByVal Value As String)
                _sOutputServer = Value
            End Set
        End Property

        Dim _sOutputLibrary As String = "WORK"
        Public Property sOutputLibrary() As String
            Get
                Return _sOutputLibrary
            End Get
            Set(ByVal Value As String)
                _sOutputLibrary = Value
            End Set
        End Property

        Dim _sOutputMemberName As String = "MERGEDATA"
        Public Property sOutputMemberName() As String
            Get
                Return _sOutputMemberName
            End Get
            Set(ByVal Value As String)
                _sOutputMemberName = Value
            End Set
        End Property

        Public Enum eMergeType
            CommonVariable
            Datasetorder
            MasterAndTransaction
            CommonVariableMergeOutputMatches
        End Enum

        Dim _eMergeType As eMergeType = eMergeType.CommonVariable
        Public Property MergeType() As eMergeType
            Get
                Return _eMergeType
            End Get
            Set(ByVal Value As eMergeType)
                If Not Value = _eMergeType Then
                    _eMergeType = Value
                End If
            End Set
        End Property
#End Region


#Region " Helper methods for serialization "
        Private Function WriteXML() As String
            Dim sw As New StringWriter
            Dim writer As New XmlTextWriter(sw)

            writer.WriteStartElement("MergeDataTask")

            ' WriteElement calls to save the state of the task
            writer.WriteElementString("ID", sID)
            writer.WriteElementString("OutputServer", sOutputServer)
            writer.WriteElementString("OutputLibrary", sOutputLibrary)
            writer.WriteElementString("OutputMemberName", sOutputMemberName)
            writer.WriteElementString("MergeType", MergeType.ToString)

            writer.WriteEndElement()
            writer.Close()

            Return sw.ToString

        End Function

        Private Function ReadXML(ByVal xml As String) As String

            If (Not (xml Is Nothing) And xml.Length > 0) Then
                Try
                    Dim sr As New StringReader(xml)
                    Dim reader As New XmlTextReader(sr)

                    reader.ReadStartElement("MergeDataTask")

                    ' ReadElement calls to read in the state of the task
                    sID = reader.ReadElementString("ID")
                    sOutputServer = reader.ReadElementString("OutputServer")
                    sOutputLibrary = reader.ReadElementString("OutputLibrary")
                    sOutputMemberName = reader.ReadElementString("OutputMemberName")

                    Dim strMergeType As String = reader.ReadElementString("MergeType")
                    Select Case strMergeType
                        Case "CommonVariable"
                            MergeType = eMergeType.CommonVariable
                        Case "CommonVariableMergeOutputMatches"
                            MergeType = eMergeType.CommonVariableMergeOutputMatches
                        Case "MasterAndTransaction"
                            MergeType = eMergeType.MasterAndTransaction
                        Case Else
                            MergeType = eMergeType.Datasetorder
                    End Select

                    reader.ReadEndElement()
                    reader.Close()
                Catch

                End Try
            End If

        End Function

#End Region


#Region " String Building Methods for SAS Code Generation "

#Region " Common Variable Merge SAS Code Generation "
        Private Function GenerateCommonVariableMergeCode() As StringBuilder
            Dim code As New StringBuilder

            Dim tc2 As ISASTaskConsumer2 = CType(_consumer, ISASTaskConsumer2)

            If Not tc2 Is Nothing Then
                Dim dsCount As Integer = tc2.InputData.Length

                ' REGION 1
                ' Define Needed SAS Macro Variables
                Dim i As Integer

                For i = 0 To dsCount - 1
                    code.Append(String.Format("%let _DS{0}={1};" & vbCrLf, i + 1, tc2.InputData(i).File))
                Next

                code.Append(String.Format("%let _ID={0};" & vbCrLf & "%let _OUTLIBRARY={1};" & vbCrLf & "%let _OUTMEMNAME={2};" & vbCrLf, sID, sOutputLibrary, sOutputMemberName))
                ' END REGION 1

                ' REGION 2
                ' Generate Sort Code
                code.Append("" & vbCrLf)
                code.Append("/* ------------------------------------------------------------------- " & vbCrLf)
                code.Append("	  Sort the data by the common ID variable... " & vbCrLf)
                code.Append("------------------------------------------------------------------- */ " & vbCrLf)
                code.Append("" & vbCrLf)

                For i = 0 To dsCount - 1
                    code.Append(String.Format("PROC SORT DATA=&_DS{0} " & vbCrLf, i + 1))
                    code.Append(String.Format(vbTab & "OUT=WORK.SORTSortedDS{0}(LABEL=\""Sorted &_DS{0}\"");" & vbCrLf, i + 1))
                    code.Append(vbTab & "BY &_ID; " & vbCrLf)
                    code.Append("RUN; " & vbCrLf)
                    code.Append("" & vbCrLf)
                Next
                ' END REGION 2

                ' REGION 3
                ' Generate Merge Code
                code.Append("/* ------------------------------------------------------------------- " & vbCrLf)
                code.Append("	Merge the Data... " & vbCrLf)
                code.Append("------------------------------------------------------------------- */ " & vbCrLf)
                code.Append("" & vbCrLf)
                code.Append("DATA &_OUTLIBRARY..&_OUTMEMNAME; " & vbCrLf)
                code.Append(vbTab & "MERGE")

                For i = 0 To dsCount - 1
                    code.Append(String.Format(" WORK.SORTSortedDS{0}", i + 1))
                Next

                code.Append("; " & vbCrLf)
                code.Append(vbTab & "BY &_ID; " & vbCrLf)
                code.Append("RUN; " & vbCrLf)
                code.Append("" & vbCrLf)
                ' END REGION 3

                ' REGION 4
                ' Generate Merge Code
                code.Append("/* ------------------------------------------------------------------- " & vbCrLf)
                code.Append("	Show the merged data... " & vbCrLf)
                code.Append("------------------------------------------------------------------- */ " & vbCrLf)
                code.Append("" & vbCrLf)
                code.Append("PROC PRINT DATA=&_OUTLIBRARY..&_OUTMEMNAME NOOBS; " & vbCrLf)
                code.Append("RUN; " & vbCrLf)

                ' END REGION 4

            End If
            Return code
        End Function
#End Region

#Region " Common Variable Merge Output Matches SAS Code Generation "
        Private Function GenerateCommonVriableMergeOutputMatchesCode() As StringBuilder
            Dim code As New StringBuilder
            Dim tc2 As ISASTaskConsumer2 = CType(_consumer, ISASTaskConsumer2)

            If Not tc2 Is Nothing Then
                Dim dsCount As Integer = tc2.InputData.Length

                ' REGION 1
                ' Define Needed SAS Macro Variables
                Dim i As Integer
                For i = 0 To dsCount - 1
                    code.Append(String.Format("%let _DS{0}={1};" & vbCrLf, i + 1, tc2.InputData(i).File))
                Next

                code.Append(String.Format("%let _ID={0};" & vbCrLf & "%let _OUTLIBRARY={1};" & vbCrLf & "%let _OUTMEMNAME={2};" & vbCrLf, sID, sOutputLibrary, sOutputMemberName))
                ' END REGION 1

                ' REGION 2
                ' Generate Sort Code
                code.Append("" & vbCrLf)
                code.Append("/* ------------------------------------------------------------------- " & vbCrLf)
                code.Append("	  Sort the data by the common ID variable... " & vbCrLf)
                code.Append("------------------------------------------------------------------- */ " & vbCrLf)
                code.Append("" & vbCrLf)

                For i = 0 To dsCount - 1
                    code.Append(String.Format("PROC SORT DATA=&_DS{0} " & vbCrLf, i + 1))
                    code.Append(String.Format(vbTab & "OUT=WORK.SORTSortedDS{0}(LABEL=\""Sorted &_DS{0}\"");" & vbCrLf, i + 1))
                    code.Append(vbTab & "BY &_ID; " & vbCrLf)
                    code.Append("RUN; " & vbCrLf)
                    code.Append("" & vbCrLf)
                Next
                ' END REGION 2

                ' REGION 3
                ' Generate Merge Code
                code.Append("/* ------------------------------------------------------------------- " & vbCrLf)
                code.Append("	Merge the Data...													" & vbCrLf)
                code.Append("	The IN= variable is a boolean flag that resets when a BY-Group		" & vbCrLf)
                code.Append("	changes. To output only matches from the MERGE, test for the		" & vbCrLf)
                code.Append("	condition that the current BY-Group is found in all data sets.		" & vbCrLf)
                code.Append("------------------------------------------------------------------- */ " & vbCrLf)
                code.Append("" & vbCrLf)
                code.Append("DATA &_OUTLIBRARY..&_OUTMEMNAME; " & vbCrLf)
                code.Append(vbTab & "MERGE")

                ' Commented in original C# code:
                ' code.Append(string.Format(" WORK.SORTSortedDS1(IN=INMASTER)"))

                For i = 0 To dsCount - 1
                    code.Append(String.Format(" WORK.SORTSortedDS{0} (IN=IN{0})", i + 1))
                Next

                code.Append("; " & vbCrLf)
                code.Append(vbTab & "BY &_ID; " & vbCrLf)

                code.Append(vbTab & "IF")

                For i = 0 To dsCount - 1
                    code.Append(String.Format(" IN{0} and", i + 1))
                Next

                code.Append(String.Format(" IN{0}; " & vbCrLf, dsCount))
                code.Append("RUN; " & vbCrLf)
                code.Append("" & vbCrLf)
                ' END REGION 3

                ' REGION 4
                ' Show the Merge Code
                code.Append("/* ------------------------------------------------------------------- " & vbCrLf)
                code.Append("	Show the merged data... " & vbCrLf)
                code.Append("------------------------------------------------------------------- */ " & vbCrLf)
                code.Append("" & vbCrLf)
                code.Append("PROC PRINT DATA=&_OUTLIBRARY..&_OUTMEMNAME NOOBS; " & vbCrLf)
                code.Append("RUN; " & vbCrLf)
                ' END REGION

            End If
            Return code
        End Function
#End Region

#Region " Master and Transaction SAS Code Generation "
        Private Function GenerateMasterAndTransactionCode() As StringBuilder
            Dim code As New StringBuilder
            Dim tc2 As ISASTaskConsumer2 = CType(_consumer, ISASTaskConsumer2)

            If Not tc2 Is Nothing Then
                Dim dsCount As Integer = tc2.InputData.Length
                Dim i As Integer

                ' REGION 1
                ' Define needed SAS Macro Variables
                For i = 0 To dsCount - 1
                    code.Append(String.Format("%let _DS{0}={1};" & vbCrLf, i + 1, tc2.InputData(i).File))
                Next

                code.Append(String.Format("%let _ID={0};" & vbCrLf & "%let _OUTLIBRARY={1};" & vbCrLf & "%let _OUTMEMNAME={2};" & vbCrLf, sID, sOutputLibrary, sOutputMemberName))
                ' END REGION 1

                ' REGION 2
                ' Generate Sort Code
                code.Append("" & vbCrLf)
                code.Append("/* ------------------------------------------------------------------- " & vbCrLf)
                code.Append("	  Sort the data by the common ID variable... " & vbCrLf)
                code.Append("------------------------------------------------------------------- */ " & vbCrLf)
                code.Append("" & vbCrLf)

                For i = 0 To dsCount - 1
                    code.Append(String.Format("PROC SORT DATA=&_DS{0} " & vbCrLf, i + 1))
                    code.Append(String.Format(vbTab & "OUT=WORK.SORTSortedDS{0}(LABEL=\""Sorted &_DS{0}\"");" & vbCrLf, i + 1))
                    code.Append(vbTab & "BY &_ID; " & vbCrLf)
                    code.Append("RUN; " & vbCrLf)
                    code.Append("" & vbCrLf)
                Next
                ' END REGION 2

                ' REGION 3
                ' Generate Merge Code
                code.Append("/* ------------------------------------------------------------------- " & vbCrLf)
                code.Append("	Merge the Data... " & vbCrLf)
                code.Append("------------------------------------------------------------------- */ " & vbCrLf)
                code.Append("" & vbCrLf)
                code.Append("DATA &_OUTLIBRARY..&_OUTMEMNAME; " & vbCrLf)
                code.Append(vbTab & "INMASTER=0;" & vbCrLf)
                code.Append(vbTab & "MERGE")

                code.Append(String.Format(" WORK.SORTSortedDS1(IN=INMASTER)"))

                For i = 0 To dsCount - 1
                    code.Append(String.Format(" WORK.SORTSortedDS{0}", i + 1))
                Next

                code.Append("; " & vbCrLf)
                code.Append(vbTab & "BY &_ID; " & vbCrLf)
                code.Append(vbTab & "IF INMASTER; " & vbCrLf)
                code.Append("RUN; " & vbCrLf)
                code.Append("" & vbCrLf)
                ' END REGION 3

                ' REGION 4
                ' Show the Merge Code
                code.Append("/* ------------------------------------------------------------------- " & vbCrLf)
                code.Append("	Show the merged data... " & vbCrLf)
                code.Append("------------------------------------------------------------------- */ " & vbCrLf)
                code.Append("" & vbCrLf)
                code.Append("PROC PRINT DATA=&_OUTLIBRARY..&_OUTMEMNAME NOOBS; " & vbCrLf)
                code.Append("RUN; " & vbCrLf)
                ' END REGION

            End If

            Return code
        End Function
#End Region

#Region " Dataset Order SAS Code Generation "
        Private Function GenerateDatasetOrderCode() As StringBuilder
            Dim code As New StringBuilder
            Dim tc2 As ISASTaskConsumer2 = CType(_consumer, ISASTaskConsumer2)

            If Not tc2 Is Nothing Then
                Dim dsCount As Integer = tc2.InputData.Length
                Dim i As Integer

                ' REGION 1
                ' Define Needed SAS Macro Variables
                For i = 0 To dsCount - 1
                    code.Append(String.Format("%let _DS{0}={1};" & vbCrLf, i + 1, tc2.InputData(i).File))
                Next
                code.Append(String.Format("%let _OUTLIBRARY={0};" & vbCrLf & "%let _OUTMEMNAME={1};" & vbCrLf, sOutputLibrary, sOutputMemberName))
                ' END REGION 1

                ' REGION 2
                ' Generate Merge Code
                code.Append("/* ------------------------------------------------------------------- " & vbCrLf)
                code.Append("	Merge the Data... " & vbCrLf)
                code.Append("------------------------------------------------------------------- */ " & vbCrLf)
                code.Append("" & vbCrLf)
                code.Append("DATA &_OUTLIBRARY..&_OUTMEMNAME; " & vbCrLf)
                code.Append(vbTab & "MERGE")

                For i = 0 To dsCount - 1
                    code.Append(String.Format(" &_DS{0}", i + 1))
                Next
                code.Append("; " & vbCrLf)
                code.Append("RUN; " & vbCrLf)
                code.Append("" & vbCrLf)
                ' END REGION 2

                ' REGION 3
                ' Show the Merge Code
                code.Append("/* ------------------------------------------------------------------- " & vbCrLf)
                code.Append("	Show the merged data... " & vbCrLf)
                code.Append("------------------------------------------------------------------- */ " & vbCrLf)
                code.Append("" & vbCrLf)
                code.Append("PROC PRINT DATA=&_OUTLIBRARY..&_OUTMEMNAME NOOBS; " & vbCrLf)
                code.Append("RUN; " & vbCrLf)
                ' END REGION

            End If

            Return code
        End Function
#End Region

#End Region


    End Class

End Namespace
