' =========================================================
'   
'                Create Picture Format Task
'
'                          a SAS Custom Task Example 
'
' Copyright (C) 2006 by SAS Institute Inc., Cary, NC 27513
'
' ---------------------------------------------------------
' The UtilityMethods class defines shared routines that are
' used from various locations throught this VB.NET project.
' They provide date, time and datetime value manipultion
' and string quoting routines.
' =========================================================

Imports System.Text

Namespace SASVB.CustomTasks

    Public Class UtilityMethods

#Region " Date, Time and DateTime Manipulating Methods "
        ' Constructs a SAS Time value from hour, minute and second values.
        Public Shared Function HMSToSASTime(ByVal hour As Int32, ByVal minute As Int32, ByVal second As Double) As Double
            Return (((hour * 60.0) + minute) * 60.0) + second
        End Function

        ' Extracts the component parts (hours, minutes and seconds) from
        ' a SAS Time value.
        Public Shared Sub SASTimeToHMS(ByVal SASTime As Double, ByRef hour As Int32, ByRef minute As Int32, ByRef second As Double)
            hour = CInt(Fix(SASTime / 3600))
            minute = CInt(Fix((CInt(Fix(SASTime)) - (hour * 3600)) / 60))
            second = SASTime - CDbl(hour * 3600 + minute * 60)
        End Sub

        ' Constructs a SAS Date value from day, month and year values.
        Public Shared Function DMYToSASDate(ByVal day As Int32, ByVal month As Int32, ByVal year As Int32) As Int32
            ' This piece of magic was addapted from some code found
            ' on from www.codeproject.com, THE web resource for .NET
            ' development.
            ' DMY to Modified Julian calculation with an extra 
            ' substraction of 2415019 and an additional 21916 to
            ' have day 0 = 01Jan1960 for SAS date values.
            Return _
                CInt(Fix((1461 * (year + 4800 + CInt(Fix((month - 14) / 12)))) / 4)) + _
                CInt(Fix((367 * (month - 2 - 12 * CInt(Fix((month - 14) / 12)))) / 12)) - _
                CInt(Fix((3 * (CInt(Fix((year + 4900 + CInt(Fix((month - 14) / 12))) / 100)))) / 4)) + _
                day - 2415019 - 32075 - 21916
        End Function

        ' Extracts the component parts (days, months and years) from
        ' a SAS Date value.
        Public Shared Sub SASDateToDMY(ByVal SASDate As Int32, ByRef day As Int32, ByRef month As Int32, ByRef year As Int32)
            ' This piece of magic was addapted from some code found
            ' on from www.codeproject.com, THE web resource for .NET
            ' development.
            ' Modified Julian to DMY calculation with an addition of
            ' 2415019 and an additional 21916 for SAS date values so
            ' that 0 = 01Jan1960.
            Dim l As Int32 = SASDate + 68569 + 2415019 + 21916
            Dim n As Int32 = CInt(Fix((4 * l) / 146097))
            l = l - CInt(Fix((146097 * n + 3) / 4))
            Dim i As Int32 = CInt(Fix((4000 * (l + 1)) / 1461001))
            l = l - CInt(Fix((1461 * i) / 4)) + 31
            Dim j As Int32 = CInt(Fix((80 * l) / 2447))
            day = l - CInt(Fix((2447 * j) / 80))
            l = CInt(Fix(j / 11))
            month = j + 2 - (12 * l)
            year = 100 * (n - 49) + i + l
        End Sub

        ' Constructs a SAS DateTime value from day, month, year,
        ' hour, minute and second values.
        Public Shared Function DMYHMSToSASDateTime(ByVal day As Int32, ByVal month As Int32, ByVal year As Int32, ByVal hour As Int32, ByVal minute As Int32, ByVal second As Double) As Double
            Return (CDbl(DMYToSASDate(day, month, year)) * 86400.0) + HMSToSASTime(hour, minute, second)
        End Function

        ' Extracts the component parts (days, months and years,
        ' hours, minutes and seconds) from a SAS DateTime value.
        Public Shared Sub SASDateTimeToDMYHMS(ByVal dateTime As Double, ByRef day As Int32, ByRef month As Int32, ByRef year As Int32, ByRef hour As Int32, ByRef minute As Int32, ByRef second As Double)
            Dim datePart As Int32 = CInt(Fix(CLng(dateTime) / CLng(86400)))
            Dim timePart As Double = dateTime - CDbl(CLng(datePart) * CLng(86400))

            SASDateToDMY(datePart, day, month, year)
            SASTimeToHMS(timePart, hour, minute, second)
        End Sub

        ' Converts a SASDate value into a string for either displaying
        ' in the UI (not eclosed in quotes with a trailing "D") or
        ' for use within SAS code (with the quotes and trailing "D").
        Public Shared Function SASDateToString(ByVal SASDate As Int32, ByVal forUseInCode As Boolean) As String
            Dim day As Int32
            Dim month As Int32
            Dim year As Int32

            ' Break the SAS Date value into its component parts.
            SASDateToDMY(SASDate, day, month, year)
            ' Now reassemble them into a string.
            Return String.Format("{0}{1}{2}{3}{4}", IIf(forUseInCode = True, """", ""), _
                                    day.ToString("D2"), _
                                    "JANFEBMARAPRMAYJUNJULAUGSEPOCTNOVDEC".Substring((month - 1) * 3, 3), _
                                    year.ToString("D4"), _
                                    IIf(forUseInCode = True, """D", ""))
        End Function

        ' Converts a SASTime value into a string for either displaying
        ' in the UI (not eclosed in quotes with a trailing "T") or
        ' for use within SAS code (with the quotes and trailing "T").
        Public Shared Function SASTimeToString(ByVal SASTime As Double, ByVal forUseInCode As Boolean) As String
            Dim hour As Int32
            Dim minute As Int32
            Dim second As Double

            ' Break the SAS Time value into its component parts.
            SASTimeToHMS(SASTime, hour, minute, second)
            ' Now reassemble them into a string.
            Return String.Format("{0}{1}:{2}:{3}{4}", IIf(forUseInCode = True, """", ""), _
                                    hour.ToString("D2"), _
                                    minute.ToString("D2"), second.ToString("00.######"), _
                                    IIf(forUseInCode = True, """T", ""))
        End Function

        ' Converts a SASDateTime value into a string for either displaying
        ' in the UI (not eclosed in quotes with a trailing "DT") or
        ' for use within SAS code (with the quotes and trailing "DT").
        Public Shared Function SASDateTimeToString(ByVal SASDateTime As Double, ByVal forUseInCode As Boolean) As String
            Dim day As Int32
            Dim month As Int32
            Dim year As Int32
            Dim hour As Int32
            Dim minute As Int32
            Dim second As Double

            ' Break the SAS DateTime value into its component parts.
            SASDateTimeToDMYHMS(SASDateTime, day, month, year, hour, minute, second)
            ' Now reassemble them into a string.
            Return String.Format("{0}{1}{2}{3}:{4}:{5}:{6}{7}", IIf(forUseInCode = True, """", ""), _
                                    day.ToString("D2"), _
                                    "JANFEBMARAPRMAYJUNJULAUGSEPOCTNOVDEC".Substring((month - 1) * 3, 3), _
                                    year.ToString("D4"), _
                                    hour.ToString("D2"), _
                                    minute.ToString("D2"), _
                                    second.ToString("00.######"), _
                                    IIf(forUseInCode = True, """DT", ""))
        End Function

        ' Takes the date part of one .NET DateTime object and the time
        ' part of another .NET DateTime object and converts them into a
        ' SAS DateTime value. 
        Public Shared Function DotNetDateAndTimeToSASDateTime(ByVal datePart As DateTime, ByVal timePart As DateTime) As Double
            Return DMYHMSToSASDateTime(datePart.Day, datePart.Month, datePart.Year, timePart.Hour, timePart.Minute, CDbl(timePart.Second) + (CDbl(timePart.Millisecond) / 1000))
        End Function

        ' Converts a .NET DateTime object into a SAS DateTime value.
        Public Shared Function DotNetDateTimeToSASDateTime(ByVal dt As DateTime) As Double
            Return DMYHMSToSASDateTime(dt.Day, dt.Month, dt.Year, dt.Hour, dt.Minute, CDbl(dt.Second) + (CDbl(dt.Millisecond) / 1000))
        End Function

        ' Converts the date portion of a .NET DateTime object into a
        ' SAS Date value.
        Public Shared Function DotNetDateTimeToSASDate(ByVal dt As DateTime) As Int32
            Return DMYToSASDate(dt.Day, dt.Month, dt.Year)
        End Function

        ' Converts the time portion of a .NET DateTime object into a
        ' SAS Time value.
        Public Shared Function DotNetDateTimeToSASTime(ByVal dt As DateTime) As Double
            Return HMSToSASTime(dt.Hour, dt.Minute, CDbl(dt.Second) + (CDbl(dt.Millisecond) / 1000))
        End Function

        ' Converts a SAS DateTime value to a .NET DateTime object.
        Public Shared Function SASDateTimeToDotNetDateTime(ByVal dt As Double) As DateTime
            Dim day As Int32
            Dim month As Int32
            Dim year As Int32
            Dim hour As Int32
            Dim minute As Int32
            Dim second As Double

            SASDateTimeToDMYHMS(dt, day, month, year, hour, minute, second)
            Return New DateTime(year, month, day, hour, minute, CInt(Fix(second)), CInt(Fix((second - CDbl(CInt(Fix(second)))) * 1000)))
        End Function

        ' Converts a SAS Date value to a .NET DateTime object.
        Public Shared Function SASDateToDotNetDateTime(ByVal d As Int32) As DateTime
            Dim day As Int32
            Dim month As Int32
            Dim year As Int32

            SASDateToDMY(d, day, month, year)
            Return New DateTime(year, month, day)
        End Function

        ' Converts a SAS Time value to a .NET DateTime object.
        Public Shared Function SASTimeToDotNetDateTime(ByVal t As Double) As DateTime
            Dim hour As Int32
            Dim minute As Int32
            Dim second As Double

            SASTimeToHMS(t, hour, minute, second)
            Return New DateTime(1960, 1, 1, hour, minute, CInt(Fix(second)), CInt(Fix((second - CDbl(CInt(Fix(second)))) * 1000)))
        End Function

        ' Drops the milliseconds value from a .NET DateTime object.
        Public Shared Function DotNetDateTimeDropMillisecs(ByVal dt As DateTime) As DateTime
            Return New DateTime(dt.Year, dt.Month, dt.Day, dt.Hour, dt.Minute, dt.Second)
        End Function
#End Region

#Region " String Quoting Methods "
        ' Quotes the supplied string by duplicating all quotes within the
        ' string and wrapping the resulting value in quotes. A flag is
        ' supplied to indicate whether single or double quotes are to be
        ' used.
        Public Shared Function QuoteString(ByVal input As String, ByVal useDoubleQuotes As Boolean) As String
            Dim output As StringBuilder = New StringBuilder
            Dim start As Int32 = 0
            Dim index As Int32
            ' Work out which quoting character we should be using.
            Dim quoteChar As String = IIf(useDoubleQuotes = True, """", "'")

            ' Start the output string with one of the appropriate
            ' quote characters.
            output.Append(quoteChar)

            ' Scan the string to find the first quote character.
            index = input.IndexOf(quoteChar)
            While (index <> -1)
                ' We appear to have found a quote character, so we
                ' append the characters from the last quote position
                ' to the new quote's position to the output string.
                output.Append(input.Substring(start, index - start + 1))
                ' Now append the 'quoting' quote.
                output.Append(quoteChar)
                ' And search from the last quote's position for the
                ' next quote.
                start = index + 1
                index = input.IndexOf(quoteChar, start)
            End While
            ' At this point, there were no more quotes found, so
            ' we need to append the left over part of the string
            ' to the output string.
            output.Append(input.Substring(start))
            ' Finish of the string with the last quoting character.
            output.Append(quoteChar)

            ' And we're done.
            Return output.ToString()
        End Function
#End Region

    End Class
End Namespace
