' ---------------------------------------------------------------
' Copyright 2006, SAS Institute Inc.
' ---------------------------------------------------------------
Imports SAS.Shared.AddIns
Imports System.Text
Imports System.Xml
Imports System.IO

Namespace SASVB.SASProducts
    Public Class SASProducts
        Implements SAS.Shared.AddIns.ISASTaskAddIn, SAS.Shared.AddIns.ISASTaskDescription, SAS.Shared.AddIns.ISASTask

        Private _consumer As ISASTaskConsumer

#Region "ISASTaskAddin"

        Public ReadOnly Property AddInDescription() As String Implements SAS.Shared.AddIns.ISASTaskAddIn.AddInDescription
            Get
                AddInDescription = "Report on what SAS products are installed and licensed on a server (Visual Basic.NET)"
            End Get
        End Property

        Public ReadOnly Property AddInName() As String Implements SAS.Shared.AddIns.ISASTaskAddIn.AddInName
            Get
                AddInName = "SAS Product Checker (VB)"
            End Get
        End Property

        Public Function Connect(ByVal inConsumer As SAS.Shared.AddIns.ISASTaskConsumer) As Boolean Implements SAS.Shared.AddIns.ISASTaskAddIn.Connect
            ' I need to call back into the consumer (host application), so I need to
            ' save the Consumer passed in for future use.
            Me._consumer = inConsumer
            Connect = True
        End Function

        Public Sub Disconnect() Implements SAS.Shared.AddIns.ISASTaskAddIn.Disconnect
            ' nothing special needed for this task
        End Sub

        Public WriteOnly Property Language() As String Implements SAS.Shared.AddIns.ISASTaskAddIn.Language
            Set(ByVal Value As String)
                ' nothing special needed for this task
            End Set
        End Property

        Public Function Languages(ByRef Items() As String) As Integer Implements SAS.Shared.AddIns.ISASTaskAddIn.Languages
            ' supporting only english in this task
            Items = New String() {"en-US"}
            Languages = 1
        End Function

        Public ReadOnly Property VisibleInManager() As Boolean Implements SAS.Shared.AddIns.ISASTaskAddIn.VisibleInManager
            Get
                VisibleInManager = True
            End Get
        End Property

#End Region

#Region "ISASTaskDescription"

        Public ReadOnly Property Clsid() As String Implements SAS.Shared.AddIns.ISASTaskDescription.Clsid
            Get
                Clsid = "5ae97039-b8be-4727-af78-c4c02ac4d5bd"
            End Get
        End Property

        Public ReadOnly Property FriendlyName() As String Implements SAS.Shared.AddIns.ISASTaskDescription.FriendlyName
            Get
                FriendlyName = "SAS Product Checker (VB)"
            End Get
        End Property

        Public ReadOnly Property GeneratesListOutput() As Boolean Implements SAS.Shared.AddIns.ISASTaskDescription.GeneratesListOutput
            Get
                GeneratesListOutput = True
            End Get
        End Property

        Public ReadOnly Property GeneratesSasCode() As Boolean Implements SAS.Shared.AddIns.ISASTaskDescription.GeneratesSasCode
            Get
                GeneratesSasCode = True
            End Get
        End Property

        Public ReadOnly Property IconAssembly() As String Implements SAS.Shared.AddIns.ISASTaskDescription.IconAssembly
            Get
                ' return this assembly as the icon assembly
                IconAssembly = System.Reflection.Assembly.GetExecutingAssembly.Location
            End Get
        End Property

        Public ReadOnly Property IconName() As String Implements SAS.Shared.AddIns.ISASTaskDescription.IconName
            Get
                IconName = "SASVB.CustomTasks.products.ico"
            End Get
        End Property

        Public ReadOnly Property MajorVersion() As Integer Implements SAS.Shared.AddIns.ISASTaskDescription.MajorVersion
            Get
                MajorVersion = 1
            End Get
        End Property

        Public ReadOnly Property MinorVersion() As Integer Implements SAS.Shared.AddIns.ISASTaskDescription.MinorVersion
            Get
                MinorVersion = 0
            End Get
        End Property

        Public ReadOnly Property NumericColsRequired() As Integer Implements SAS.Shared.AddIns.ISASTaskDescription.NumericColsRequired
            Get
                NumericColsRequired = 0
            End Get
        End Property

        Public ReadOnly Property ProcsUsed() As String Implements SAS.Shared.AddIns.ISASTaskDescription.ProcsUsed
            Get
                ProcsUsed = "DATA step"
            End Get
        End Property

        Public ReadOnly Property ProductsOptional() As String Implements SAS.Shared.AddIns.ISASTaskDescription.ProductsOptional
            Get
                ProductsOptional = ""
            End Get
        End Property

        Public ReadOnly Property ProductsRequired() As String Implements SAS.Shared.AddIns.ISASTaskDescription.ProductsRequired
            Get
                ProductsRequired = "BASE"
            End Get
        End Property

        Public ReadOnly Property RequiresData() As Boolean Implements SAS.Shared.AddIns.ISASTaskDescription.RequiresData
            Get
                RequiresData = False
            End Get
        End Property

        Public ReadOnly Property StandardCategory() As Boolean Implements SAS.Shared.AddIns.ISASTaskDescription.StandardCategory
            Get
                StandardCategory = False
            End Get
        End Property

        Public ReadOnly Property TaskCategory() As String Implements SAS.Shared.AddIns.ISASTaskDescription.TaskCategory
            Get
                TaskCategory = Global.Category
            End Get
        End Property

        Public ReadOnly Property TaskDescription() As String Implements SAS.Shared.AddIns.ISASTaskDescription.TaskDescription
            Get
                TaskDescription = "Report on what SAS products are installed and licensed on a server (Visual Basic.NET)"
            End Get
        End Property

        Public ReadOnly Property TaskName() As String Implements SAS.Shared.AddIns.ISASTaskDescription.TaskName
            Get
                TaskName = "SAS Products Checker (VB)"
            End Get
        End Property

        Public ReadOnly Property TaskType() As SAS.Shared.AddIns.ShowType Implements SAS.Shared.AddIns.ISASTaskDescription.TaskType
            Get
                TaskType = ShowType.Wizard
            End Get
        End Property

        Public ReadOnly Property Validation() As String Implements SAS.Shared.AddIns.ISASTaskDescription.Validation
            Get
                Validation = "SAS-supplied example"
            End Get
        End Property

        Public ReadOnly Property WhatIsDescription() As String Implements SAS.Shared.AddIns.ISASTaskDescription.WhatIsDescription
            Get
                WhatIsDescription = "Report on what SAS products are installed and licensed on a server (Visual Basic.NET)"
            End Get
        End Property

#End Region

#Region "ISASTask"

        Public Function Initialize() As Boolean Implements SAS.Shared.AddIns.ISASTask.Initialize
            Initialize = True
        End Function

        Public Property Label() As String Implements SAS.Shared.AddIns.ISASTask.Label
            Get
                Label = "SAS Products Checker (VB)"
            End Get
            Set(ByVal Value As String)

            End Set
        End Property

        Public ReadOnly Property OutputDataCount() As Integer Implements SAS.Shared.AddIns.ISASTask.OutputDataCount
            Get
                OutputDataCount = 0
            End Get
        End Property

        Public Function OutputDataInfo(ByVal Index As Integer, ByRef Source As String, ByRef Label As String) As SAS.Shared.AddIns.OutputData Implements SAS.Shared.AddIns.ISASTask.OutputDataInfo
            Source = Nothing
            Label = Nothing
            OutputDataInfo = OutputData.Unknown
        End Function

        Public ReadOnly Property RunLog() As String Implements SAS.Shared.AddIns.ISASTask.RunLog
            Get
                RunLog = Nothing
            End Get
        End Property

        Public ReadOnly Property SasCode() As String Implements SAS.Shared.AddIns.ISASTask.SasCode
            Get
                Dim text1 As String = Global.ReadFileFromAssembly("SASVB.CustomTasks.sasproducts.sas")
                Dim text2 As String = Global.ReadFileFromAssembly("SASVB.CustomTasks.sasproductsrep.sas")
                Return (text1 & text2)
            End Get
        End Property

        Public Function Show(ByVal Owner As System.Windows.Forms.IWin32Window) As SAS.Shared.AddIns.ShowResult Implements SAS.Shared.AddIns.ISASTask.Show

            ' show the form
            Dim form1 As New SASProductsForm
            form1.Consumer = Me._consumer
            form1.ShowDialog(Owner)

            ' if the user clicked the Report button, return RunNow status
            If form1.CreateReport Then
                Me._consumer.AssignedServer = form1.CurrentServer
                Return ShowResult.RunNow
            End If

            ' otherwise we'll just return Canceled
            Return ShowResult.Canceled
        End Function

        Public Sub Terminate() Implements SAS.Shared.AddIns.ISASTask.Terminate
            ' nothing special needed for this task
        End Sub

        ' this task does not save any state
        Public Property XmlState() As String Implements SAS.Shared.AddIns.ISASTask.XmlState
            Get
            End Get

            Set(ByVal Value As String)
            End Set
        End Property

#End Region

    End Class

End Namespace
